@@ -15,12 +15,16 @@
 
 #include <thread>	// for std::this_thread::sleep_for
 #include <chrono>	// for std::chrono::milliseconds
+#include <algorithm>
+#include <cmath>
 #include <unistd.h>
 
 #include "FFmpegUtilities.h"
+#include "effects/CropHelpers.h"
 
 #include "FFmpegReader.h"
 #include "Exceptions.h"
+#include "MemoryTrim.h"
 #include "Timeline.h"
 #include "ZmqLogger.h"
 
@@ -69,13 +73,16 @@ int hw_de_on = 0;
 #endif
 
 FFmpegReader::FFmpegReader(const std::string &path, bool inspect_reader)
+		: FFmpegReader(path, DurationStrategy::VideoPreferred, inspect_reader) {}
+
+FFmpegReader::FFmpegReader(const std::string &path, DurationStrategy duration_strategy, bool inspect_reader)
 		: last_frame(0), is_seeking(0), seeking_pts(0), seeking_frame(0), seek_count(0), NO_PTS_OFFSET(-99999),
 		  path(path), is_video_seek(true), check_interlace(false), check_fps(false), enable_seek(true), is_open(false),
 		  seek_audio_frame_found(0), seek_video_frame_found(0),is_duration_known(false), largest_frame_processed(0),
-		  current_video_frame(0), packet(NULL), max_concurrent_frames(OPEN_MP_NUM_PROCESSORS), audio_pts(0),
-		  video_pts(0), pFormatCtx(NULL), videoStream(-1), audioStream(-1), pCodecCtx(NULL), aCodecCtx(NULL),
-		  pStream(NULL), aStream(NULL), pFrame(NULL), previous_packet_location{-1,0},
-		  hold_packet(false) {
+		  current_video_frame(0), packet(NULL), duration_strategy(duration_strategy),
+		  audio_pts(0), video_pts(0), pFormatCtx(NULL), videoStream(-1), audioStream(-1), pCodecCtx(NULL), aCodecCtx(NULL),
+		pStream(NULL), aStream(NULL), pFrame(NULL), previous_packet_location{-1,0},
+		hold_packet(false) {
 
 	// Initialize FFMpeg, and register all formats and codecs
 	AV_REGISTER_ALL
@@ -87,8 +94,8 @@ FFmpegReader::FFmpegReader(const std::string &path, bool inspect_reader)
 	audio_pts_seconds = NO_PTS_OFFSET;
 
 	// Init cache
-	working_cache.SetMaxBytesFromInfo(max_concurrent_frames * info.fps.ToDouble() * 2, info.width, info.height, info.sample_rate, info.channels);
-	final_cache.SetMaxBytesFromInfo(max_concurrent_frames * 2, info.width, info.height, info.sample_rate, info.channels);
+	working_cache.SetMaxBytesFromInfo(info.fps.ToDouble() * 2, info.width, info.height, info.sample_rate, info.channels);
+	final_cache.SetMaxBytesFromInfo(24, info.width, info.height, info.sample_rate, info.channels);
 
 	// Open and Close the reader, to populate its attributes (such as height, width, etc...)
 	if (inspect_reader) {
@@ -278,7 +285,7 @@ void FFmpegReader::Open() {
 				retry_decode_open = 0;
 
 				// Set number of threads equal to number of processors (not to exceed 16)
-				pCodecCtx->thread_count = std::min(FF_NUM_PROCESSORS, 16);
+				pCodecCtx->thread_count = std::min(FF_VIDEO_NUM_PROCESSORS, 16);
 
 				if (pCodec == NULL) {
 					throw InvalidCodec("A valid video codec could not be found for this file.", path);
@@ -524,8 +531,8 @@ void FFmpegReader::Open() {
 			const AVCodec *aCodec = avcodec_find_decoder(codecId);
 			aCodecCtx = AV_GET_CODEC_CONTEXT(aStream, aCodec);
 
-			// Set number of threads equal to number of processors (not to exceed 16)
-			aCodecCtx->thread_count = std::min(FF_NUM_PROCESSORS, 16);
+			// Audio encoding does not typically use more than 2 threads (most codecs use 1 thread)
+			aCodecCtx->thread_count = std::min(FF_AUDIO_NUM_PROCESSORS, 2);
 
 			if (aCodec == NULL) {
 				throw InvalidCodec("A valid audio codec could not be found for this file.", path);
@@ -554,13 +561,59 @@ void FFmpegReader::Open() {
 			info.metadata[str_key.toStdString()] = str_value.trimmed().toStdString();
 		}
 
+		// Process video stream side data (rotation, spherical metadata, etc)
+		for (unsigned int i = 0; i < pFormatCtx->nb_streams; i++) {
+			AVStream* st = pFormatCtx->streams[i];
+			if (st->codecpar->codec_type == AVMEDIA_TYPE_VIDEO) {
+				// Only inspect the first video stream
+				for (int j = 0; j < st->nb_side_data; j++) {
+					AVPacketSideData *sd = &st->side_data[j];
+
+					// Handle rotation metadata (unchanged)
+					if (sd->type == AV_PKT_DATA_DISPLAYMATRIX &&
+						sd->size >= 9 * sizeof(int32_t) &&
+						!info.metadata.count("rotate"))
+					{
+						double rotation = -av_display_rotation_get(
+							reinterpret_cast<int32_t *>(sd->data));
+						if (std::isnan(rotation)) rotation = 0;
+						info.metadata["rotate"] = std::to_string(rotation);
+					}
+					// Handle spherical video metadata
+					else if (sd->type == AV_PKT_DATA_SPHERICAL) {
+						// Always mark as spherical
+						info.metadata["spherical"] = "1";
+
+						// Cast the raw bytes to an AVSphericalMapping
+						const AVSphericalMapping* map =
+							reinterpret_cast<const AVSphericalMapping*>(sd->data);
+
+						// Projection enum → string
+						const char* proj_name = av_spherical_projection_name(map->projection);
+						info.metadata["spherical_projection"] = proj_name
+							? proj_name
+							: "unknown";
+
+						// Convert 16.16 fixed-point to float degrees
+						auto to_deg = [](int32_t v){
+							return (double)v / 65536.0;
+						};
+						info.metadata["spherical_yaw"]   = std::to_string(to_deg(map->yaw));
+						info.metadata["spherical_pitch"] = std::to_string(to_deg(map->pitch));
+						info.metadata["spherical_roll"]  = std::to_string(to_deg(map->roll));
+					}
+				}
+				break;
+			}
+		}
+
 		// Init previous audio location to zero
 		previous_packet_location.frame = -1;
 		previous_packet_location.sample_start = 0;
 
 		// Adjust cache size based on size of frame and audio
-		working_cache.SetMaxBytesFromInfo(max_concurrent_frames * info.fps.ToDouble() * 2, info.width, info.height, info.sample_rate, info.channels);
-		final_cache.SetMaxBytesFromInfo(max_concurrent_frames * 2, info.width, info.height, info.sample_rate, info.channels);
+		working_cache.SetMaxBytesFromInfo(info.fps.ToDouble() * 2, info.width, info.height, info.sample_rate, info.channels);
+		final_cache.SetMaxBytesFromInfo(24, info.width, info.height, info.sample_rate, info.channels);
 
 		// Scan PTS for any offsets (i.e. non-zero starting streams). At least 1 stream must start at zero timestamp.
 		// This method allows us to shift timestamps to ensure at least 1 stream is starting at zero.
@@ -632,6 +685,13 @@ void FFmpegReader::Close() {
 				}
 			}
 #endif // USE_HW_ACCEL
+			if (img_convert_ctx) {
+				sws_freeContext(img_convert_ctx);
+				img_convert_ctx = nullptr;
+			}
+			if (pFrameRGB_cached) {
+				AV_FREE_FRAME(&pFrameRGB_cached);
+			}
 		}
 
 		// Close the audio codec
@@ -640,6 +700,11 @@ void FFmpegReader::Close() {
 				avcodec_flush_buffers(aCodecCtx);
 			}
 			AV_FREE_CONTEXT(aCodecCtx);
+			if (avr_ctx) {
+				SWR_CLOSE(avr_ctx);
+				SWR_FREE(&avr_ctx);
+				avr_ctx = nullptr;
+			}
 		}
 
 		// Clear final cache
@@ -650,6 +715,9 @@ void FFmpegReader::Close() {
 		avformat_close_input(&pFormatCtx);
 		av_freep(&pFormatCtx);
 
+		// Release free’d arenas back to OS after heavy teardown
+		TrimMemoryToOS(true);
+
 		// Reset some variables
 		last_frame = 0;
 		hold_packet = false;
@@ -669,6 +737,74 @@ bool FFmpegReader::HasAlbumArt() {
 		&& (pFormatCtx->streams[videoStream]->disposition & AV_DISPOSITION_ATTACHED_PIC);
 }
 
+double FFmpegReader::PickDurationSeconds() const {
+	auto has_value = [](double value) { return value > 0.0; };
+
+	switch (duration_strategy) {
+		case DurationStrategy::VideoPreferred:
+			if (has_value(video_stream_duration_seconds))
+				return video_stream_duration_seconds;
+			if (has_value(audio_stream_duration_seconds))
+				return audio_stream_duration_seconds;
+			if (has_value(format_duration_seconds))
+				return format_duration_seconds;
+			break;
+		case DurationStrategy::AudioPreferred:
+			if (has_value(audio_stream_duration_seconds))
+				return audio_stream_duration_seconds;
+			if (has_value(video_stream_duration_seconds))
+				return video_stream_duration_seconds;
+			if (has_value(format_duration_seconds))
+				return format_duration_seconds;
+			break;
+		case DurationStrategy::LongestStream:
+		default:
+			{
+				double longest = 0.0;
+				if (has_value(video_stream_duration_seconds))
+					longest = std::max(longest, video_stream_duration_seconds);
+				if (has_value(audio_stream_duration_seconds))
+					longest = std::max(longest, audio_stream_duration_seconds);
+				if (has_value(format_duration_seconds))
+					longest = std::max(longest, format_duration_seconds);
+				if (has_value(longest))
+					return longest;
+			}
+			break;
+	}
+
+	if (has_value(format_duration_seconds))
+		return format_duration_seconds;
+	if (has_value(inferred_duration_seconds))
+		return inferred_duration_seconds;
+
+	return 0.0;
+}
+
+void FFmpegReader::ApplyDurationStrategy() {
+	const double fps_value = info.fps.ToDouble();
+	const double chosen_seconds = PickDurationSeconds();
+
+	if (chosen_seconds <= 0.0 || fps_value <= 0.0) {
+		info.duration = 0.0f;
+		info.video_length = 0;
+		is_duration_known = false;
+		return;
+	}
+
+	const int64_t frames = static_cast<int64_t>(std::llround(chosen_seconds * fps_value));
+	if (frames <= 0) {
+		info.duration = 0.0f;
+		info.video_length = 0;
+		is_duration_known = false;
+		return;
+	}
+
+	info.video_length = frames;
+	info.duration = static_cast<float>(static_cast<double>(frames) / fps_value);
+	is_duration_known = true;
+}
+
 void FFmpegReader::UpdateAudioInfo() {
 	// Set default audio channel layout (if needed)
 #if HAVE_CH_LAYOUT
@@ -684,6 +820,11 @@ void FFmpegReader::UpdateAudioInfo() {
 		return;
 	}
 
+	auto record_duration = [](double &target, double seconds) {
+		if (seconds > 0.0)
+			target = std::max(target, seconds);
+	};
+
 	// Set values of FileInfo struct
 	info.has_audio = true;
 	info.file_size = pFormatCtx->pb ? avio_size(pFormatCtx->pb) : -1;
@@ -717,34 +858,27 @@ void FFmpegReader::UpdateAudioInfo() {
 	info.audio_timebase.den = aStream->time_base.den;
 
 	// Get timebase of audio stream (if valid) and greater than the current duration
-	if (aStream->duration > 0 && aStream->duration > info.duration) {
-		// Get duration from audio stream
-		info.duration = aStream->duration * info.audio_timebase.ToDouble();
-	} else if (pFormatCtx->duration > 0 && info.duration <= 0.0f) {
-		// Use the format's duration
-		info.duration = float(pFormatCtx->duration) / AV_TIME_BASE;
+	if (aStream->duration > 0) {
+		record_duration(audio_stream_duration_seconds, aStream->duration * info.audio_timebase.ToDouble());
+	}
+	if (pFormatCtx->duration > 0) {
+		// Use the format's duration when stream duration is missing or shorter
+		record_duration(format_duration_seconds, static_cast<double>(pFormatCtx->duration) / AV_TIME_BASE);
 	}
 
 	// Calculate duration from filesize and bitrate (if any)
 	if (info.duration <= 0.0f && info.video_bit_rate > 0 && info.file_size > 0) {
 		// Estimate from bitrate, total bytes, and framerate
-		info.duration = float(info.file_size) / info.video_bit_rate;
-	}
-
-	// Check for an invalid video length
-	if (info.has_video && info.video_length <= 0) {
-		// Calculate the video length from the audio duration
-		info.video_length = info.duration * info.fps.ToDouble();
+		record_duration(inferred_duration_seconds, static_cast<double>(info.file_size) / info.video_bit_rate);
 	}
 
 	// Set video timebase (if no video stream was found)
 	if (!info.has_video) {
 		// Set a few important default video settings (so audio can be divided into frames)
-		info.fps.num = 24;
+		info.fps.num = 30;
 		info.fps.den = 1;
 		info.video_timebase.num = 1;
-		info.video_timebase.den = 24;
-		info.video_length = info.duration * info.fps.ToDouble();
+		info.video_timebase.den = 30;
 		info.width = 720;
 		info.height = 480;
 
@@ -759,10 +893,7 @@ void FFmpegReader::UpdateAudioInfo() {
 		}
 	}
 
-	// Fix invalid video lengths for certain types of files (MP3 for example)
-	if (info.has_video && ((info.duration * info.fps.ToDouble()) - info.video_length > 60)) {
-		info.video_length = info.duration * info.fps.ToDouble();
-	}
+	ApplyDurationStrategy();
 
 	// Add audio metadata (if any found)
 	AVDictionaryEntry *tag = NULL;
@@ -779,6 +910,11 @@ void FFmpegReader::UpdateVideoInfo() {
 		return;
 	}
 
+	auto record_duration = [](double &target, double seconds) {
+		if (seconds > 0.0)
+			target = std::max(target, seconds);
+	};
+
 	// Set values of FileInfo struct
 	info.has_video = true;
 	info.file_size = pFormatCtx->pb ? avio_size(pFormatCtx->pb) : -1;
@@ -854,50 +990,35 @@ void FFmpegReader::UpdateVideoInfo() {
 	info.video_timebase.den = pStream->time_base.den;
 
 	// Set the duration in seconds, and video length (# of frames)
-	info.duration = pStream->duration * info.video_timebase.ToDouble();
+	record_duration(video_stream_duration_seconds, pStream->duration * info.video_timebase.ToDouble());
 
 	// Check for valid duration (if found)
-	if (info.duration <= 0.0f && pFormatCtx->duration >= 0) {
-		// Use the format's duration
-		info.duration = float(pFormatCtx->duration) / AV_TIME_BASE;
+	if (pFormatCtx->duration >= 0) {
+		// Use the format's duration as another candidate
+		record_duration(format_duration_seconds, static_cast<double>(pFormatCtx->duration) / AV_TIME_BASE);
 	}
 
 	// Calculate duration from filesize and bitrate (if any)
-	if (info.duration <= 0.0f && info.video_bit_rate > 0 && info.file_size > 0) {
+	if (info.video_bit_rate > 0 && info.file_size > 0) {
 		// Estimate from bitrate, total bytes, and framerate
-		info.duration = float(info.file_size) / info.video_bit_rate;
+		record_duration(inferred_duration_seconds, static_cast<double>(info.file_size) / info.video_bit_rate);
 	}
 
 	// Certain "image" formats do not have a valid duration
-	if (info.duration <= 0.0f && pStream->duration == AV_NOPTS_VALUE && pFormatCtx->duration == AV_NOPTS_VALUE) {
+	if (video_stream_duration_seconds <= 0.0 && format_duration_seconds <= 0.0 &&
+			pStream->duration == AV_NOPTS_VALUE && pFormatCtx->duration == AV_NOPTS_VALUE) {
 		// Force an "image" duration
-		info.duration = 60 * 60 * 1;  // 1 hour duration
-		info.video_length = 1;
+		record_duration(video_stream_duration_seconds, 60 * 60 * 1);  // 1 hour duration
+		info.has_single_image = true;
+	}
+	// Static GIFs can have no usable duration; fall back to a small default
+	if (video_stream_duration_seconds <= 0.0 && format_duration_seconds <= 0.0 &&
+			pFormatCtx && pFormatCtx->iformat && strcmp(pFormatCtx->iformat->name, "gif") == 0) {
+		record_duration(video_stream_duration_seconds, 60 * 60 * 1);  // 1 hour duration
 		info.has_single_image = true;
 	}
 
-	// Get the # of video frames (if found in stream)
-	// Only set this 1 time (this method can be called multiple times)
-	if (pStream->nb_frames > 0 && info.video_length <= 0) {
-		info.video_length = pStream->nb_frames;
-	}
-
-	// No duration found in stream of file
-	if (info.duration <= 0.0f) {
-		// No duration is found in the video stream
-		info.duration = -1;
-		info.video_length = -1;
-		is_duration_known = false;
-	} else {
-		// Yes, a duration was found
-		is_duration_known = true;
-
-		// Calculate number of frames (if not already found in metadata)
-		// Only set this 1 time (this method can be called multiple times)
-		if (info.video_length <= 0) {
-			info.video_length = round(info.duration * info.fps.ToDouble());
-		}
-	}
+	ApplyDurationStrategy();
 
 	// Add video metadata (if any)
 	AVDictionaryEntry *tag = NULL;
@@ -985,7 +1106,7 @@ std::shared_ptr<Frame> FFmpegReader::ReadStream(int64_t requested_frame) {
 	int packet_error = -1;
 
 	// Debug output
-	ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::ReadStream", "requested_frame", requested_frame, "max_concurrent_frames", max_concurrent_frames);
+	ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::ReadStream", "requested_frame", requested_frame);
 
 	// Loop through the stream until the correct frame is found
 	while (true) {
@@ -1269,7 +1390,10 @@ bool FFmpegReader::GetAVFrame() {
 			frameFinished = 1;
 			packet_status.video_decoded++;
 
-			av_image_alloc(pFrame->data, pFrame->linesize, info.width, info.height, (AVPixelFormat)(pStream->codecpar->format), 1);
+			// Allocate image (align 32 for simd)
+			if (AV_ALLOCATE_IMAGE(pFrame, (AVPixelFormat)(pStream->codecpar->format), info.width, info.height) <= 0) {
+				throw OutOfMemory("Failed to allocate image buffer", path);
+			}
 			av_image_copy(pFrame->data, pFrame->linesize, (const uint8_t**)next_frame->data, next_frame->linesize,
 										(AVPixelFormat)(pStream->codecpar->format), info.width, info.height);
 
@@ -1407,15 +1531,17 @@ void FFmpegReader::ProcessVideoPacket(int64_t requested_frame) {
 	int width = info.width;
 	int64_t video_length = info.video_length;
 
-	// Create variables for a RGB Frame (since most videos are not in RGB, we must convert it)
-	AVFrame *pFrameRGB = nullptr;
+	// Create or reuse a RGB Frame (since most videos are not in RGB, we must convert it)
+	AVFrame *pFrameRGB = pFrameRGB_cached;
+	if (!pFrameRGB) {
+		pFrameRGB = AV_ALLOCATE_FRAME();
+		if (pFrameRGB == nullptr)
+			throw OutOfMemory("Failed to allocate frame buffer", path);
+		pFrameRGB_cached = pFrameRGB;
+	}
+	AV_RESET_FRAME(pFrameRGB);
 	uint8_t *buffer = nullptr;
 
-	// Allocate an AVFrame structure
-	pFrameRGB = AV_ALLOCATE_FRAME();
-	if (pFrameRGB == nullptr)
-		throw OutOfMemory("Failed to allocate frame buffer", path);
-
 	// Determine the max size of this source image (based on the timeline's size, the scaling mode,
 	// and the scaling keyframes). This is a performance improvement, to keep the images as small as possible,
 	// without losing quality. NOTE: We cannot go smaller than the timeline itself, or the add_layer timeline
@@ -1469,6 +1595,9 @@ void FFmpegReader::ProcessVideoPacket(int64_t requested_frame) {
 			max_width = info.width * max_scale_x * preview_ratio;
 			max_height = info.height * max_scale_y * preview_ratio;
 		}
+
+		// If a crop effect is resizing the image, request enough pixels to preserve detail
+		ApplyCropResizeScale(parent, info.width, info.height, max_width, max_height);
 	}
 
 	// Determine if image needs to be scaled (for performance reasons)
@@ -1492,8 +1621,12 @@ void FFmpegReader::ProcessVideoPacket(int64_t requested_frame) {
 
 	// Determine required buffer size and allocate buffer
 	const int bytes_per_pixel = 4;
-	int buffer_size = (width * height * bytes_per_pixel) + 128;
-	buffer = new unsigned char[buffer_size]();
+	int raw_buffer_size = (width * height * bytes_per_pixel) + 128;
+
+	// Aligned memory allocation (for speed)
+	constexpr size_t ALIGNMENT = 32;  // AVX2
+	int buffer_size = ((raw_buffer_size + ALIGNMENT - 1) / ALIGNMENT) * ALIGNMENT;
+	buffer = (unsigned char*) aligned_malloc(buffer_size, ALIGNMENT);
 
 	// Copy picture data from one AVFrame (or AVPicture) to another one.
 	AV_COPY_PICTURE_DATA(pFrameRGB, buffer, PIX_FMT_RGBA, width, height);
@@ -1502,8 +1635,9 @@ void FFmpegReader::ProcessVideoPacket(int64_t requested_frame) {
 	if (openshot::Settings::Instance()->HIGH_QUALITY_SCALING) {
 		scale_mode = SWS_BICUBIC;
 	}
-	SwsContext *img_convert_ctx = sws_getContext(info.width, info.height, AV_GET_CODEC_PIXEL_FORMAT(pStream, pCodecCtx), width,
-												 height, PIX_FMT_RGBA, scale_mode, NULL, NULL, NULL);
+	img_convert_ctx = sws_getCachedContext(img_convert_ctx, info.width, info.height, AV_GET_CODEC_PIXEL_FORMAT(pStream, pCodecCtx), width, height, PIX_FMT_RGBA, scale_mode, NULL, NULL, NULL);
+	if (!img_convert_ctx)
+		throw OutOfMemory("Failed to initialize sws context", path);
 
 	// Resize / Convert to RGB
 	sws_scale(img_convert_ctx, pFrame->data, pFrame->linesize, 0,
@@ -1528,11 +1662,10 @@ void FFmpegReader::ProcessVideoPacket(int64_t requested_frame) {
 	last_video_frame = f;
 
 	// Free the RGB image
-	AV_FREE_FRAME(&pFrameRGB);
+	AV_RESET_FRAME(pFrameRGB);
 
-	// Remove frame and packet
-	RemoveAVFrame(pFrame);
-	sws_freeContext(img_convert_ctx);
+    // Remove frame and packet
+    RemoveAVFrame(pFrame);
 
 	// Get video PTS in seconds
 	video_pts_seconds = (double(video_pts) * info.video_timebase.ToDouble()) + pts_offset_seconds;
@@ -1676,10 +1809,10 @@ void FFmpegReader::ProcessAudioPacket(int64_t requested_frame) {
 	audio_converted->nb_samples = audio_frame->nb_samples;
 	av_samples_alloc(audio_converted->data, audio_converted->linesize, info.channels, audio_frame->nb_samples, AV_SAMPLE_FMT_FLTP, 0);
 
-	SWRCONTEXT *avr = NULL;
-
-	// setup resample context
-	avr = SWR_ALLOC();
+	SWRCONTEXT *avr = avr_ctx;
+	// setup resample context if needed
+	if (!avr) {
+		avr = SWR_ALLOC();
 #if HAVE_CH_LAYOUT
 	av_opt_set_chlayout(avr, "in_chlayout", &AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->ch_layout, 0);
 	av_opt_set_chlayout(avr, "out_chlayout", &AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->ch_layout, 0);
@@ -1694,6 +1827,8 @@ void FFmpegReader::ProcessAudioPacket(int64_t requested_frame) {
 	av_opt_set_int(avr, "in_sample_rate", info.sample_rate, 0);
 	av_opt_set_int(avr, "out_sample_rate", info.sample_rate, 0);
 	SWR_INIT(avr);
+	avr_ctx = avr;
+	}
 
 	// Convert audio samples
 	int nb_samples = SWR_CONVERT(avr,	// audio resample context
@@ -1704,10 +1839,6 @@ void FFmpegReader::ProcessAudioPacket(int64_t requested_frame) {
 							 audio_frame->linesize[0],	   // input plane size, in bytes (0 if unknown)
 							 audio_frame->nb_samples);	   // number of input samples to convert
 
-	// Deallocate resample buffer
-	SWR_CLOSE(avr);
-	SWR_FREE(&avr);
-	avr = NULL;
 
 	int64_t starting_frame_number = -1;
 	for (int channel_filter = 0; channel_filter < info.channels; channel_filter++) {
@@ -1816,7 +1947,7 @@ void FFmpegReader::Seek(int64_t requested_frame) {
 	seek_count++;
 
 	// If seeking near frame 1, we need to close and re-open the file (this is more reliable than seeking)
-	int buffer_amount = std::max(max_concurrent_frames, 8);
+	int buffer_amount = 12;
 	if (requested_frame - buffer_amount < 20) {
 		// prevent Open() from seeking again
 		is_seeking = true;
@@ -2385,6 +2516,18 @@ Json::Value FFmpegReader::JsonValue() const {
 	Json::Value root = ReaderBase::JsonValue(); // get parent properties
 	root["type"] = "FFmpegReader";
 	root["path"] = path;
+	switch (duration_strategy) {
+		case DurationStrategy::VideoPreferred:
+			root["duration_strategy"] = "VideoPreferred";
+			break;
+		case DurationStrategy::AudioPreferred:
+			root["duration_strategy"] = "AudioPreferred";
+			break;
+		case DurationStrategy::LongestStream:
+		default:
+			root["duration_strategy"] = "LongestStream";
+			break;
+	}
 
 	// return JsonValue
 	return root;
@@ -2414,6 +2557,16 @@ void FFmpegReader::SetJsonValue(const Json::Value root) {
 	// Set data from Json (if key is found)
 	if (!root["path"].isNull())
 		path = root["path"].asString();
+	if (!root["duration_strategy"].isNull()) {
+		const std::string strategy = root["duration_strategy"].asString();
+		if (strategy == "VideoPreferred") {
+			duration_strategy = DurationStrategy::VideoPreferred;
+		} else if (strategy == "AudioPreferred") {
+			duration_strategy = DurationStrategy::AudioPreferred;
+		} else {
+			duration_strategy = DurationStrategy::LongestStream;
+		}
+	}
 
 	// Re-Open path, and re-init everything (if needed)
 	if (is_open) {
